<?php
/**
 * @file
 * Form definitions for admin forms.
 */

/**
 * Basic edit form for the content entity.
 *
 * The entity being edited should be stored in $form_state['entity']
 * when this form is built.
 *
 * The name of this form is autogenerated as $entity_type . '_edit_form'.
 */
function cps_review_edit_form($form, &$form_state) {
  $entity = $form_state['entity'];

  $form['entity'] = array();
  $form['entity']['title'] = array(
    '#type' => 'textfield',
    '#title' => t('Subject'),
    '#description' => t('Review Title'),
    '#default_value' => $entity->title,
    '#access' => variable_get('cps_review_use_title', 0) == 1,
    '#weight' => -10,
  );
  $form['entity']['body'] = array();

  $entity_type = $entity->entityType();
  field_attach_form($entity_type, $entity, $form['entity']['body'], $form_state, entity_language($entity_type, $entity));

  $form['actions'] = array(
    '#type' => 'actions',
  );

  $form['actions']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
  );

  return $form;
}

/**
 * Validate callback for the content entity.
 */
function cps_review_edit_form_validate($form, &$form_state) {
  $entity = $form_state['entity'];
  field_attach_form_validate($entity->entityType(), $entity, $form, $form_state);
}

/**
 * Submit callback for the content entity.
 */
function cps_review_edit_form_submit($form, &$form_state) {
  $entity = $form_state['entity'];
  entity_form_submit_build_entity('cps_review', $entity, $form, $form_state);
  _cps_review_populate_title($entity);
}

/**
 * Ensures a title is always set and possibly calculated from the body.
 *
 * @param $entity
 *   The entity to populate the title property on if empty.
 */
function _cps_review_populate_title($entity) {
  if (trim($entity->title) != '') {
    return;
  }

  // The body may be in any format, so:
  // 1) Filter it into HTML
  // 2) Strip out all HTML tags
  // 3) Convert entities back to plain-text.
  $field = field_info_field('cps_review_body');
  $langcode = field_is_translatable('cps_review_body', $field) ? entity_language('cps_review', $comment) : LANGUAGE_NONE;
  $body = $entity->cps_review_body[$langcode][0];
  if (isset($body['format'])) {
    $text = check_markup($body['value'], $body['format']);
  }
  else {
    $text = check_plain($body['value']);
  }
  $entity->title = truncate_utf8(trim(decode_entities(strip_tags($text))), 29, TRUE);

  // Edge cases where the entity body is populated only by HTML tags will
  // require a default title.
  if ($entity->title == '') {
    $entity->title = t('(No title)');
  }
}


/**
 * Form callback to display the publish form.
 */
function cps_review_publish_form($form, &$form_state) {
  $form['warning'] = array(
    '#markup' => '<div class="publish-warning">' . t('Are you sure you want to publish this?') . '</div>',
  );

  $form['actions']['publish'] = array(
    '#type' => 'submit',
    '#value' => t('Publish'),
  );

  $form['actions']['cancel'] = array(
    '#type' => 'submit',
    '#value' => t('Cancel'),
    '#limit_validation_errors' => array(),
    '#submit' => array('cps_review_publish_form_cancel'),
  );

  return $form;
}

/**
 * Submit handler to cancel the form.
 */
function cps_review_publish_form_cancel($form, &$form_state) {
  $form_state['redirect'] = $form_state['entity']->uri();
}

/**
 * Submit handler for the publish form.
 */
function cps_review_publish_form_submit($form, &$form_state) {
  $entity = $form_state['entity'];
  $entity->status = ENTITY_PUBLISHED;
  $entity->save();
  $form_state['redirect'] = $form_state['entity']->uri();
}

// -----------------------------------------------------------------------
// Unpublish form and handlers.

/**
 * Form callback to display the unpublish form.
 */
function cps_review_unpublish_form($form, &$form_state) {
  $entity = $form_state['entity'];

  $form['warning'] = array(
    '#markup' => '<div class="unpublish-warning">' . t('Are you sure you want to unpublish this?') . '</div>',
  );

  $form['actions']['unpublish'] = array(
    '#type' => 'submit',
    '#value' => t('Unpublish'),
  );

  $form['actions']['cancel'] = array(
    '#type' => 'submit',
    '#value' => t('Cancel'),
    '#limit_validation_errors' => array(),
    // We re-use the publish_form_cancel here because all it does is redirect.
    '#submit' => array('cps_review_publish_form_cancel'),
  );

  return $form;
}

/**
 * Submit handler for the unpublish form.
 */
function cps_review_unpublish_form_submit($form, &$form_state) {
  $entity = $form_state['entity'];
  $entity->status = ENTITY_UNPUBLISHED;
  $entity->save();
  $form_state['redirect'] = $form_state['entity']->uri();
}
